@@ -10,18 +10,18 @@
 namespace ltp {
 namespace ner {
 
-using namespace ltp::utility;
-
 class Extractor {
 public:
   static Extractor& extractor();
   static int num_templates();
-  static int extract1o(Instance * inst, int idx, std::vector< StringVec > & cache);
+  static int extract1o(const Instance& inst, int idx,
+      std::vector<utility::StringVec> & cache);
+
 protected:
   Extractor();
   ~Extractor();
 private:
-  static std::vector< Template * > templates;
+  static std::vector<utility::Template *> templates;
 };
 
 }       //  end for namespace segmentor
