@@ -13,7 +13,7 @@
     #define POSTAGGER_DLL_API extern "C" _declspec(dllexport)
 #else
     #define POSTAGGER_DLL_API extern "C" _deslspec(dllimport)
-    #pragma comment(lib, "postagger.lib")
+    //#pragma comment(lib, "postagger.lib")
 #endif // end for PARSER_DLL_API
 #endif // end for _WIN32
 
