@@ -5,11 +5,15 @@
 #include <algorithm>
 #include <vector>
 #include <cstring>
+#include "boost/cstdint.hpp"
 #include "utils/hasher.hpp"
 
 namespace ltp {
 namespace utility {
 
+using boost::uint32_t;
+using boost::int32_t;
+
 struct __SmartMap_Hash_Node {
 public:
   unsigned int    __key_off;
@@ -29,7 +33,7 @@ public:
   }
 };
 
-template <class T = int>
+template <class T = int32_t>
 struct __SmartMap_Const_Iterator {
   typedef __SmartMap_Hash_Node hash_node_t;
 
@@ -46,9 +50,9 @@ struct __SmartMap_Const_Iterator {
     key_buffer(0),
     val_buffer(0) {}
 
-  const char * key() { return key_buffer + ptr->__key_off; }
-  const T * value() { return val_buffer + ptr->__val_off; }
-  int frequency() { return ptr->__freq; }
+  const char * key() const { return key_buffer + ptr->__key_off; }
+  const T * value() const { return val_buffer + ptr->__val_off; }
+  int frequency() const { return ptr->__freq; }
   bool operator ==(const __SmartMap_Const_Iterator & other) const { return ptr == other.ptr; }
   bool operator !=(const __SmartMap_Const_Iterator & other) const { return ptr != other.ptr; }
   void operator ++() { ++ ptr; }
@@ -58,8 +62,8 @@ struct __SmartMap_Const_Iterator {
   const T *           val_buffer;
 };
 
-template <class T = int, 
-         class HashFunction = __Default_CharArray_HashFunction, 
+template <class T = int32_t,
+         class HashFunction = __Default_CharArray_HashFunction,
          class StringEqual  = __Default_CharArray_EqualFunction>
 class SmartMap {
 public:
@@ -83,12 +87,12 @@ public:
     _cap_buckets    = PRIMES[_cap_buckets_idx];
     _max_buckets    = int(0.7 * _cap_buckets);
     // allocate memory
-    _hash_buckets   = new int[ _cap_buckets ];
+    _hash_buckets   = new int32_t[ _cap_buckets ];
     _hash_buffer    = new hash_node_t[_cap_entries];
     _key_buffer     = new char[_cap_key_buffer];
     _val_buffer     = new T[_cap_entries];
 
-    _hash_buckets_volumn = new int[ _cap_buckets ];
+    _hash_buckets_volumn = new int32_t[ _cap_buckets ];
 
     // set the hash_table to be empty
     for (unsigned i = 0; i < _cap_buckets; ++ i) {
@@ -131,8 +135,8 @@ public:
         // not find this hash node
         _append(key, val, hv, idx);
 
-        for (p = _hash_buckets[idx]; 
-            _hash_buffer[p].__next_off >= 0; 
+        for (p = _hash_buckets[idx];
+            _hash_buffer[p].__next_off >= 0;
             (p = _hash_buffer[p].__next_off));
 
         _hash_buffer[p].__next_off = (_latest_hash_node - _hash_buffer);
@@ -164,14 +168,14 @@ public:
       _max_buckets = int(_cap_buckets * 0.7);
 
       // allocate a new bucket
-      int * new_hash_buckets_volumn = new int[_cap_buckets];
-      int * new_hash_buckets = new int[_cap_buckets];
-      for (unsigned i = 0; i < _cap_buckets; ++ i) {
+      int32_t * new_hash_buckets_volumn = new int32_t[_cap_buckets];
+      int32_t* new_hash_buckets = new int32_t[_cap_buckets];
+      for (unsigned int i = 0; i < _cap_buckets; ++ i) {
         new_hash_buckets[i] = -1;
         new_hash_buckets_volumn[i] = 0;
       }
 
-      for (unsigned i = 0; i < _num_entries; ++ i) {
+      for (unsigned int i = 0; i < _num_entries; ++ i) {
         unsigned int hash_val = _hash_buffer[i].__hash_val;
         unsigned int bucket_id = (hash_val % _cap_buckets);
         int freq = _hash_buffer[i].__freq;
@@ -223,9 +227,9 @@ public:
    *  @param[out] val   the value
    *  @return   bool  true on key exist, otherwise false
    */
-  bool get(const char * key, T & val) {
-    unsigned hv = HashFunction()(key);
-    unsigned idx = (hv % _cap_buckets);
+  bool get(const char * key, T & val) const {
+    unsigned int hv = HashFunction()(key);
+    unsigned int idx = (hv % _cap_buckets);
     int p = _find(key, hv, idx, false);
 
     if (-1 == p) {
@@ -244,9 +248,9 @@ public:
    *  @return   T *   pointer to the value when key exist,
    *            otherwise NULL
    */
-  T * get(const char * key) {
-    unsigned hv = HashFunction()(key);
-    unsigned idx = (hv % _cap_buckets);
+  T * get(const char * key) const {
+    unsigned int hv = HashFunction()(key);
+    unsigned int idx = (hv % _cap_buckets);
     int p = _find(key, hv, idx, false);
 
     if (-1 == p) {
@@ -264,7 +268,7 @@ public:
    *  @param[in]  key   the key
    *  @return   int   the frequency
    */
-  int frequency(const char * key) {
+  int frequency(const char * key) const {
     unsigned hv = HashFunction()(key);
     unsigned idx = (hv % _cap_buckets);
     int p = _find(key, hv, idx, false);
@@ -285,7 +289,7 @@ public:
    *  @return   bool    true on this key exist, otherwise
    *              false.
    */
-  bool contains(const char * key, bool add_freq = false) {
+  bool contains(const char * key, bool add_freq = false) const {
     unsigned int hv = HashFunction()(key);
     unsigned int idx = (hv % _cap_buckets);
 
@@ -331,11 +335,11 @@ public:
     return _num_entries;
   }
 
-  const_iterator begin() {
+  const_iterator begin() const {
     return const_iterator(_hash_buffer, _key_buffer, _val_buffer);
   }
 
-  const_iterator end() {
+  const_iterator end() const {
     return const_iterator(_hash_buffer + _num_entries, _key_buffer, _val_buffer);
   }
 
@@ -345,7 +349,7 @@ public:
    *
    *  @param[in/out]  out   the output file stream
    */
-  void dump(std::ostream & out) {
+  void dump(std::ostream & out) const {
     // write header information
     int header_size = 0;
     char header[4] = {'S', 'M', 'A', 'P'};
@@ -355,10 +359,10 @@ public:
     out.write(header, 4);
     header_size += 4;
 
-    out.write(reinterpret_cast<const char *>(&_num_entries),  sizeof(unsigned int));
-    out.write(reinterpret_cast<const char *>(&_len_key_buffer), sizeof(unsigned int));
-    out.write(reinterpret_cast<const char *>(&_cap_buckets),  sizeof(unsigned int));
-    out.write(reinterpret_cast<const char *>(_hash_buckets),  sizeof(int) * _cap_buckets);
+    out.write(reinterpret_cast<const char *>(&_num_entries),  sizeof(uint32_t));
+    out.write(reinterpret_cast<const char *>(&_len_key_buffer), sizeof(uint32_t));
+    out.write(reinterpret_cast<const char *>(&_cap_buckets),  sizeof(uint32_t));
+    out.write(reinterpret_cast<const char *>(_hash_buckets),  sizeof(int32_t) * _cap_buckets);
     out.write(reinterpret_cast<const char *>(_hash_buffer),   sizeof(hash_node_t) * _num_entries);
     out.write(reinterpret_cast<const char *>(_key_buffer),    sizeof(char) * _len_key_buffer);
     out.write(reinterpret_cast<const char *>(_val_buffer),    sizeof(T) * _num_entries);
@@ -381,17 +385,17 @@ public:
       return false;
     }
 
-    in.read(reinterpret_cast<char *>(&_num_entries),  sizeof(unsigned int));
-    in.read(reinterpret_cast<char *>(&_len_key_buffer), sizeof(unsigned int));
-    in.read(reinterpret_cast<char *>(&_cap_buckets),  sizeof(unsigned int));
+    in.read(reinterpret_cast<char *>(&_num_entries),  sizeof(uint32_t));
+    in.read(reinterpret_cast<char *>(&_len_key_buffer), sizeof(uint32_t));
+    in.read(reinterpret_cast<char *>(&_cap_buckets),  sizeof(uint32_t));
 
-    _hash_buckets   = new int[_cap_buckets];
+    _hash_buckets   = new int32_t[_cap_buckets];
     _hash_buffer  = new hash_node_t[_num_entries];
     _key_buffer   = new char[_len_key_buffer];
     _val_buffer   = new T[_num_entries];
 
 
-    in.read(reinterpret_cast<char *>(_hash_buckets),  sizeof(int) * _cap_buckets);
+    in.read(reinterpret_cast<char *>(_hash_buckets),  sizeof(uint32_t) * _cap_buckets);
     in.read(reinterpret_cast<char *>(_hash_buffer),   sizeof(hash_node_t) * _num_entries);
     in.read(reinterpret_cast<char *>(_key_buffer),    sizeof(char) * _len_key_buffer);
     in.read(reinterpret_cast<char *>(_val_buffer),    sizeof(T) * _num_entries);
@@ -438,8 +442,8 @@ protected:
   static const unsigned int PRIMES[100]; 
 
 protected:
-  int *       _hash_buckets;
-  int *       _hash_buckets_volumn;
+  int32_t *       _hash_buckets;
+  int32_t *       _hash_buckets_volumn;
   hash_node_t *   _hash_buffer;
   char *      _key_buffer;  /*< the buffer of key */
   T *       _val_buffer;  /*< the buffer of value */
@@ -460,10 +464,10 @@ protected:
   unsigned int  _len_key_buffer;
   unsigned int  _cap_key_buffer;
 
-  unsigned int  _father;
+  // unsigned int  _father;
 
   char *      _latest_key;
-  T *       _latest_val;
+  T *         _latest_val;
   hash_node_t *   _latest_hash_node;
 
 protected:
@@ -550,7 +554,7 @@ protected:
    *              is added when the hash entry is found
    *  @return   int     the position of the hash node in hsh buffer
    */
-  int _find(const char * key, unsigned hv, unsigned idx, bool add_freq) {
+  int _find(const char * key, unsigned hv, unsigned idx, bool add_freq) const {
     int p = _hash_buckets[idx];
 
     while (p >= 0) {
@@ -564,7 +568,7 @@ protected:
           }
           return p;
         } else {
-          _father = p;
+          // _father = p;
           p = _hash_buffer[p].__next_off;
         }
       }
@@ -584,7 +588,7 @@ const unsigned int SmartMap<T, HashFunction, StringEqual>::PRIMES[100] = {
   1610612741,
 };
 
-class IndexableSmartMap : public SmartMap<int> {
+class IndexableSmartMap : public SmartMap<int32_t> {
 public:
   IndexableSmartMap() : entries(0), cap_entries(0) {}
 
@@ -595,8 +599,8 @@ public:
   }
 
 private:
-  int cap_entries;
-  int * entries;
+  uint32_t cap_entries;
+  int32_t* entries;
 
 public:
   /**
@@ -608,32 +612,32 @@ public:
   // offsets of the key in hashmap key buffer is stored in entries.
   // when a new key is insert into the collection, check if entries
   // buffer is big enough. if not, duplicate the memory.
-  int push(const char * key) {
-    if (!SmartMap<int>::contains(key)) {
-      int idx = SmartMap<int>::size();
+  int32_t push(const char * key) {
+    if (!SmartMap<int32_t>::contains(key)) {
+      int32_t idx = SmartMap<int32_t>::size();
       set(key, idx);
 
-      if (cap_entries < SmartMap<int>::_num_entries) {
-        cap_entries = ( SmartMap<int>::_num_entries << 1);
-        int * new_entries = new int[cap_entries];
+      if (cap_entries < SmartMap<int32_t>::_num_entries) {
+        cap_entries = ( SmartMap<int32_t>::_num_entries << 1);
+        int32_t* new_entries = new int32_t[cap_entries];
         if ( entries ) {
-          memcpy(new_entries, entries, sizeof(int) * (SmartMap<int>::_num_entries - 1));
+          memcpy(new_entries, entries, sizeof(int32_t) * (SmartMap<int32_t>::_num_entries - 1));
           delete [](entries);
         }
         entries = new_entries;
       }
 
       // SmartMap<int>::debug(cout);
-      entries[_num_entries-1] = SmartMap<int>::_latest_hash_node->__key_off;
+      entries[_num_entries-1] = SmartMap<int32_t>::_latest_hash_node->__key_off;
       return idx;
     } else {
-      return (*SmartMap<int>::get(key));
+      return (*SmartMap<int32_t>::get(key));
     }
 
     return -1;
   }
 
-  int push(const std::string & key) {
+  int32_t push(const std::string & key) {
     return push(key.c_str());
   }
 
@@ -643,9 +647,9 @@ public:
    *  @param[in]  i         the index
    *  @return   const char *  pointer to the key
    */
-  const char * at(int i) {
-    if (i >= 0 && i < _num_entries) {
-      return SmartMap<int>::_key_buffer + entries[i];
+  const char* at(const size_t& i) const {
+    if (i < _num_entries) {
+      return SmartMap<int32_t>::_key_buffer + entries[i];
     } else {
       return 0;
     }
@@ -657,16 +661,16 @@ public:
    *  @param[in]  key       the key
    *  @return   int       index of the key if exist, otherwise -1
    */
-  int index( const char * key ) {
-    int val = -1;
-    if (SmartMap<int>::get(key, val)) {
+  int32_t index(const char * key) const {
+    int32_t val = -1;
+    if (SmartMap<int32_t>::get(key, val)) {
       return val;
     }
 
     return -1;
   }
 
-  int index( const std::string & key) {
+  int index(const std::string & key) const {
     return index(key.c_str());
   }
 
@@ -675,9 +679,9 @@ public:
    *
    *  @param[out] out   the output stream
    */
-  void dump(std::ostream & out) {
-    SmartMap<int>::dump(out);
-    out.write(reinterpret_cast<const char *>(entries), sizeof(int) * _num_entries);
+  void dump(std::ostream & out) const {
+    SmartMap<int32_t>::dump(out);
+    out.write(reinterpret_cast<const char *>(entries), sizeof(int32_t) * _num_entries);
   }
 
   /**
@@ -687,7 +691,7 @@ public:
    *  @return   bool  true on success, otherwise false
    */
   bool load(std::istream & in) {
-    bool ret = SmartMap<int>::load(in);
+    bool ret = SmartMap<int32_t>::load(in);
     if (!ret) {
       return ret;
     }
@@ -696,12 +700,12 @@ public:
       delete [](entries);
     }
 
-    entries = new int[SmartMap<int>::_num_entries];
+    entries = new int32_t[SmartMap<int32_t>::_num_entries];
     if (!entries) {
       return false;
     }
 
-    in.read(reinterpret_cast<char *>(entries), sizeof(int) * _num_entries);
+    in.read(reinterpret_cast<char *>(entries), sizeof(int32_t) * _num_entries);
     return true;
   }
 };
